@@ -84,12 +84,16 @@
 #if _POSIX_TIMERS > 0
 #include "dasynq/posixtimer.h"
 namespace dasynq {
+inline namespace v2 {
     template <typename T, bool provide_mono_timer = true> using timer_events = posix_timer_events<T, provide_mono_timer>;
+} // namespace v2
 } // namespace dasynq
 #else
 #include "dasynq/itimer.h"
 namespace dasynq {
+inline namespace v2 {
     template <typename T, bool provide_mono_timer = true> using timer_events = itimer_events<T, provide_mono_timer>;
+} // namespace v2
 } // namespace dasynq
 #endif
 #endif
@@ -99,15 +103,19 @@ namespace dasynq {
 #include "dasynq/kqueue-macos.h"
 #include "dasynq/childproc.h"
 namespace dasynq {
+inline namespace v2 {
     template <typename T> using loop_t = macos_kqueue_loop<timer_events<child_proc_events<interrupt_channel<T>>, false>>;
     using loop_traits_t = macos_kqueue_traits;
+} // namespace v2
 } // namespace dasynq
 #else
 #include "dasynq/kqueue.h"
 #include "dasynq/childproc.h"
 namespace dasynq {
+inline namespace v2 {
     template <typename T> using loop_t = kqueue_loop<timer_events<child_proc_events<interrupt_channel<T>>, false>>;
     using loop_traits_t = kqueue_traits;
+} // namespace v2
 } // namespace dasynq
 #endif
 #elif DASYNQ_HAVE_EPOLL
@@ -115,22 +123,28 @@ namespace dasynq {
 #include "dasynq/timerfd.h"
 #include "dasynq/childproc.h"
 namespace dasynq {
+inline namespace v2 {
     template <typename T> using loop_t = epoll_loop<interrupt_channel<timer_fd_events<child_proc_events<T>>>>;
     using loop_traits_t = epoll_traits;
+} // namespace v2
 } // namespace dasynq
 #else
 #include "dasynq/childproc.h"
 #if DASYNQ_HAVE_PSELECT
 #include "dasynq/pselect.h"
 namespace dasynq {
+inline namespace v2 {
     template <typename T> using loop_t = pselect_events<timer_events<interrupt_channel<child_proc_events<T>>, false>>;
     using loop_traits_t = select_traits;
+} // namespace v2
 } // namespace dasynq
 #else
 #include "dasynq/select.h"
 namespace dasynq {
+inline namespace v2 {
     template <typename T> using loop_t = select_events<timer_events<interrupt_channel<child_proc_events<T>>, false>>;
     using loop_traits_t = select_traits;
+} // namespace v2
 } // namespace dasynq
 #endif
 #endif
@@ -174,6 +188,7 @@ class delayed_init {
     DASYNQ_EMPTY_BODY
 };
 
+inline namespace v2 {
 namespace dprivate {
 
     // Classes for implementing a fair(ish) wait queue.
@@ -425,7 +440,7 @@ namespace dprivate {
     // lock; they are also free to use it to protect their own internal data structures.
     template <typename Traits, typename LoopTraits> class event_dispatch
     {
-        friend class dasynq::event_loop<typename LoopTraits::mutex_t, LoopTraits>;;
+        friend class dasynq::event_loop<typename LoopTraits::mutex_t, LoopTraits>;
 
         public:
         using mutex_t = typename LoopTraits::mutex_t;
@@ -438,7 +453,7 @@ namespace dprivate {
         prio_queue event_queue;
         
         using base_signal_watcher = dprivate::base_signal_watcher<typename traits_t::sigdata_t>;
-        using base_child_watcher = dprivate::base_child_watcher;
+        using base_child_watcher = dprivate::base_child_watcher<typename traits_t::proc_status_t>;
         using base_timer_watcher = dprivate::base_timer_watcher;
         
         // Add a watcher into the queueing system (but don't queue it). Call with lock held.
@@ -535,7 +550,7 @@ namespace dprivate {
         }
         
         // Child process terminated. Called with both the main lock and the reaper lock held.
-        void receive_child_stat(pid_t child, int status, void * userdata) noexcept
+        void receive_child_stat(pid_t child, typename LoopTraits::backend_traits_t::proc_status_t status, void * userdata) noexcept
         {
             base_child_watcher * watcher = static_cast<base_child_watcher *>(userdata);
             watcher->child_status = status;
@@ -635,7 +650,7 @@ namespace dprivate {
         event_dispatch(const event_dispatch &) = delete;
     };
 
-} // namespace dasynq
+} // namespace dprivate
 
 // This is the main event_loop implementation. It serves as an interface to the event loop backend (of which
 // it maintains an internal instance). It also serialises polling the backend and provides safe deletion of
@@ -681,7 +696,7 @@ class event_loop
     using base_signal_watcher = dprivate::base_signal_watcher<typename loop_traits_t::sigdata_t>;
     using base_fd_watcher = dprivate::base_fd_watcher;
     using base_bidi_fd_watcher = dprivate::base_bidi_fd_watcher;
-    using base_child_watcher = dprivate::base_child_watcher;
+    using base_child_watcher = dprivate::base_child_watcher<typename loop_traits_t::proc_status_t>;
     using base_timer_watcher = dprivate::base_timer_watcher;
     using watch_type_t = dprivate::watch_type_t;
 
@@ -705,8 +720,8 @@ class event_loop
     // To solve that, we:
     // - allow only one thread to poll for events at a time, using a lock
     // - use the same lock to prevent polling, if we want to unwatch an event source
-    // - generate an event to interrupt any polling that may already be occurring in
-    //   another thread
+    // - generate an event to interrupt, when necessary, any polling that may already be occurring
+    //   in another thread
     // - mark handlers as active if they are currently executing, and
     // - when removing an active handler, simply set a flag which causes it to be
     //   removed once the current processing is finished, rather than removing it
@@ -740,6 +755,9 @@ class event_loop
     //    - if the node is at the head of the queue, lock is claimed; return
     //    - otherwise, if a poll is in progress, interrupt it
     //    - wait until our node is at the head of the attn_waitqueue
+    //
+    // Some backends also need to interrupted in order to add a new watch (eg select/pselect).
+    // However, the attn_waitqueue lock doesn't generally need to be obtained for this.
     
     mutex_t wait_lock;  // protects the wait/attention queues
     bool long_poll_running = false;  // whether any thread is polling the backend (with non-zero timeout)
@@ -1021,28 +1039,28 @@ class event_loop
         }
     }
     
-    void set_timer(base_timer_watcher *callBack, const timespec &timeout, clock_type clock) noexcept
+    void set_timer(base_timer_watcher *callback, const timespec &timeout, clock_type clock) noexcept
     {
         struct timespec interval {0, 0};
-        loop_mech.set_timer(callBack->timer_handle, timeout, interval, true, clock);
+        loop_mech.set_timer(callback->timer_handle, timeout, interval, true, clock);
     }
     
-    void set_timer(base_timer_watcher *callBack, const timespec &timeout, const timespec &interval,
+    void set_timer(base_timer_watcher *callback, const timespec &timeout, const timespec &interval,
             clock_type clock) noexcept
     {
-        loop_mech.set_timer(callBack->timer_handle, timeout, interval, true, clock);
+        loop_mech.set_timer(callback->timer_handle, timeout, interval, true, clock);
     }
 
-    void set_timer_rel(base_timer_watcher *callBack, const timespec &timeout, clock_type clock) noexcept
+    void set_timer_rel(base_timer_watcher *callback, const timespec &timeout, clock_type clock) noexcept
     {
         struct timespec interval {0, 0};
-        loop_mech.set_timer_rel(callBack->timer_handle, timeout, interval, true, clock);
+        loop_mech.set_timer_rel(callback->timer_handle, timeout, interval, true, clock);
     }
     
-    void set_timer_rel(base_timer_watcher *callBack, const timespec &timeout,
+    void set_timer_rel(base_timer_watcher *callback, const timespec &timeout,
             const timespec &interval, clock_type clock) noexcept
     {
-        loop_mech.set_timer_rel(callBack->timer_handle, timeout, interval, true, clock);
+        loop_mech.set_timer_rel(callback->timer_handle, timeout, interval, true, clock);
     }
 
     void set_timer_enabled(base_timer_watcher *callback, clock_type clock, bool enabled) noexcept
@@ -2034,7 +2052,7 @@ class bidi_fd_watcher_impl : public bidi_fd_watcher<EventLoop>
 
 // Child process event watcher
 template <typename EventLoop>
-class child_proc_watcher : private dprivate::base_child_watcher
+class child_proc_watcher : private dprivate::base_child_watcher<typename EventLoop::loop_traits_t::proc_status_t>
 {
     template <typename, typename> friend class child_proc_watcher_impl;
 
@@ -2044,13 +2062,14 @@ class child_proc_watcher : private dprivate::base_child_watcher
     public:
 
     using event_loop_t = EventLoop;
+    using proc_status_t = typename EventLoop::loop_traits_t::proc_status_t;
 
     // send a signal to this process, if it is still running, in a race-free manner.
     // return is as for POSIX kill(); return is -1 with errno=ESRCH if process has
     // already terminated.
     int send_signal(event_loop_t &loop, int signo) noexcept
     {
-        auto reaper_mutex = loop.get_reaper_mutex();
+        auto reaper_mutex = loop.get_reaper_lock();
         std::lock_guard<decltype(reaper_mutex)> guard(reaper_mutex);
 
         if (this->child_termd) {
@@ -2202,7 +2221,7 @@ class child_proc_watcher : private dprivate::base_child_watcher
         }
     }
     
-    // virtual rearm child_status(EventLoop &eloop, pid_t child, int status) = 0;
+    // virtual rearm child_status(EventLoop &eloop, pid_t child, proc_status_t status) = 0;
 };
 
 template <typename EventLoop, typename Derived>
@@ -2367,6 +2386,7 @@ class timer_impl : public timer<EventLoop>
 
 } // namespace dprivate
 
+} // namespace v2
 } // namespace dasynq
 
 #endif /* DASYNQ_H_ */
