@@ -15,7 +15,98 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "baseproc-sys.h"
+#include <baseproc-sys.h>
+
+// Check if a value is one of several possible values.
+// Use like:     value(x).is_in(1,2,3)
+template <typename T>
+class value_cls {
+    const T &v;
+public:
+    value_cls(const T &v) : v(v) {}
+
+    template <typename U>
+    bool is_in(U&& val)
+    {
+        return v == val;
+    }
+
+    template <typename U, typename ...V>
+    bool is_in(U&&val, V&&... vals) {
+        if (v == val)
+            return true;
+        return is_in(vals...);
+    }
+};
+
+template <typename T>
+value_cls<T> value(const T &v)
+{
+    return value_cls<T>(v);
+}
+
+// Since we are C++11, we don't have a std::string_view.
+class string_view
+{
+    const char *s = nullptr;
+    size_t count = 0;
+
+public:
+    string_view() = default;
+    string_view(const string_view &other) = default;
+    string_view(const char *s_p, size_t count_p) noexcept : s(s_p), count(count_p) { }
+    string_view(const char *s_p) noexcept  : s(s_p), count(strlen(s_p)) { }
+
+    string_view(const std::string &str) : s(str.data()), count(str.length()) { }
+
+    string_view &operator=(const string_view &other) = default;
+
+    bool operator==(const string_view &other) const noexcept
+    {
+        if (count != other.count) return false;
+        return memcmp(s, other.s, count) == 0;
+    }
+
+    bool operator==(const char *other) const noexcept
+    {
+        if (strncmp(s, other, count) == 0) {
+            if (other[count] == '\0') {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    const char *data() const noexcept { return s; }
+    size_t size() const noexcept { return count; }
+    size_t length() const noexcept { return count; }
+    bool empty() const noexcept { return count == 0; }
+    string_view substr(size_t begin, size_t len) const noexcept
+    {
+        return string_view(s + begin, std::min(len, count - begin));
+    }
+
+    const char *begin() const noexcept { return s; }
+    const char *end() const noexcept { return s + count; }
+    std::reverse_iterator<const char *> rbegin() const noexcept { return std::reverse_iterator<const char *>(end()); }
+    std::reverse_iterator<const char *> rend() const noexcept { return std::reverse_iterator<const char *>(begin()); }
+
+    size_t find(char c) const noexcept {
+        auto it = std::find(begin(), end(), c);
+        if (it == end()) return std::string::npos;
+        return (it - begin());
+    }
+    size_t rfind(char c) const noexcept {
+        auto it = std::find(rbegin(), rend(), c);
+        if (it == rend()) return std::string::npos;
+        return (&(*it) - begin());
+    }
+
+    operator std::string() const
+    {
+        return std::string(s, count);
+    }
+};
 
 // Complete read - read the specified size until end-of-file or error; continue read if
 // interrupted by signal.
@@ -48,31 +139,31 @@ inline ssize_t complete_read(int fd, void * buf, size_t n)
 
 // Combine two paths to produce a path. If the second path is absolute, it is returned unmodified;
 // otherwise, it is appended to the first path (with a slash separator added if needed).
-inline std::string combine_paths(const std::string &p1, const char * p2)
+inline std::string combine_paths(string_view p1, const char * p2)
 {
-    if (*p2 == 0) return p1;
+    if (*p2 == 0) return (std::string)p1;
     if (p1.empty()) return std::string(p2);
 
     if (p2[0] == '/') return p2;
 
-    if (*(p1.rbegin()) == '/') return p1 + p2;
-    return p1 + '/' + p2;
+    if (*(p1.rbegin()) == '/') return (std::string)p1 + p2;
+    return (std::string)p1 + '/' + p2;
 }
 
 // Find the parent path of a given path, which should refer to a named file or directory (not . or ..).
 // If the path contains no directory, returns the empty string.
-inline std::string parent_path(const std::string &p)
+inline string_view parent_path(string_view p)
 {
     auto spos = p.rfind('/');
     if (spos == std::string::npos) {
-        return std::string {};
+        return string_view {};
     }
 
     return p.substr(0, spos + 1);
 }
 
 // Find the base name of a path (the name after the final '/').
-inline const char *base_name(const char *path)
+inline const char *base_name(const char *path) noexcept
 {
     const char * basen = path;
     const char * s = path;
@@ -84,7 +175,7 @@ inline const char *base_name(const char *path)
 }
 
 // Check if one string starts with another
-inline bool starts_with(const std::string &s, const char *prefix)
+inline bool starts_with(const std::string &s, const char *prefix) noexcept
 {
     const char *sp = s.c_str();
     while (*sp != 0 && *prefix != 0) {
@@ -94,7 +185,8 @@ inline bool starts_with(const std::string &s, const char *prefix)
     return *prefix == 0;
 }
 
-// An allocator that doesn't value-initialise for construction
+// An allocator that doesn't value-initialise for construction. Eg for containers of primitive types this
+// allocator avoids the overhead of initialising new elements to 0.
 template <typename T>
 class default_init_allocator : public std::allocator<T>
 {
@@ -106,59 +198,20 @@ public:
         using other = default_init_allocator<U>;
     };
 
-    // Note this is only a template so that if there is no suitable constructor for T, we won't
-    // error out here
-    template <typename U>
-    void construct(U *obj)
+    template <typename U = std::enable_if<std::is_default_constructible<T>::value>>
+    void construct(T *obj)
     {
-        ::new(obj) U;
+        // avoid value-initialisation:
+        ::new(obj) T;
     }
 
     template <typename ...Args>
-    void construct(T *obj, Args... args)
+    void construct(T *obj, Args&&... args)
     {
-        std::allocator<T>::construct(obj, args...);
+        std::allocator<T>::construct(obj, std::forward<Args>(args)...);
     }
 };
 
-// Since we are C++11, we don't have a std::string_view.
-class string_view
-{
-    const char *s = nullptr;
-    size_t count = 0;
-
-public:
-    string_view() = default;
-    string_view(const string_view &other) = default;
-    string_view(const char *s_p, size_t count_p) : s(s_p), count(count_p) { }
-    string_view(const char *s_p) : s(s_p), count(strlen(s_p)) { }
-
-    string_view(const std::string &str) : s(str.data()), count(str.length()) { }
-
-    string_view &operator=(const string_view &other) = default;
-
-    bool operator==(const string_view &other) const
-    {
-        if (count != other.count) return false;
-        return memcmp(s, other.s, count) == 0;
-    }
-
-    bool operator==(const char *other) const
-    {
-        if (strncmp(s, other, count) == 0) {
-            if (other[count] == '\0') {
-                return true;
-            }
-        }
-        return false;
-    }
-
-    const char *data() const { return s; }
-    size_t size() const { return count; }
-    size_t length() const { return count; }
-    bool empty() const { return count == 0; }
-};
-
 inline size_t hash(const string_view &str)
 {
     size_t end_pos = str.length();
@@ -197,9 +250,9 @@ class dinit_equal_to
 {
 public:
     template <typename A, typename B>
-    bool operator()(const A &a, const B &b)
+    bool operator()(A &&a, B &&b)
     {
-        return a == b;
+        return std::forward<A>(a) == std::forward<B>(b);
     }
 };
 
@@ -353,7 +406,7 @@ private:
             // First, check if the value is already present
             bucket_num = hashval % buckets.size();
             auto list_it = std::find_if(buckets[bucket_num].begin(), buckets[bucket_num].end(),
-                    [&](key_type &k) { return key_equal_f(k,value); });
+                    [&](const key_type &k) { return key_equal_f(k,std::forward<V>(value)); });
             if (list_it != buckets[bucket_num].end()) {
                 return { { &buckets, list_it, bucket_num }, false };
             }
