@@ -41,7 +41,8 @@ enum class shutdown_type_t {
     REMAIN,            // Continue running with no services
     HALT,              // Halt system without powering down
     POWEROFF,          // Power off system
-    REBOOT             // Reboot system
+    REBOOT,            // Reboot system
+    SOFTREBOOT         // Reboot dinit
 };
 
 /* Reasons for why service stopped */
@@ -125,6 +126,13 @@ enum class log_type_id
     PIPE      // pipe to another process (service)
 };
 
+enum class auto_restart_mode
+{
+    NEVER,      // Never automatically restart
+    ALWAYS,     // Always restart
+    ON_FAILURE, // Only restart when process fails
+};
+
 // Service set type identifiers:
 constexpr int SSET_TYPE_NONE = 0;
 constexpr int SSET_TYPE_DIRLOAD = 1;
