@@ -118,7 +118,7 @@ rearm exec_status_pipe_watcher::fd_event(eventloop_t &loop, int fd, int flags) n
 
         if (sr->pid == -1) {
             // Somehow the process managed to complete before we even saw the exec() status.
-            sr->handle_exit_status(sr->exit_status);
+            sr->handle_exit_status();
         }
     }
 
@@ -209,12 +209,13 @@ rearm ready_notify_watcher::fd_event(eventloop_t &, int fd, int flags) noexcept
     return rearm::REARM;
 }
 
-dasynq::rearm service_child_watcher::status_change(eventloop_t &loop, pid_t child, int status) noexcept
+dasynq::rearm service_child_watcher::status_change(eventloop_t &loop, pid_t child,
+        service_child_watcher::proc_status_t status) noexcept
 {
     base_process_service *sr = service;
 
     sr->pid = -1;
-    sr->exit_status = bp_sys::exit_status(status);
+    sr->exit_status = status;
 
     // Ok, for a process service, any process death which we didn't rig ourselves is a bit... unexpected.
     // Probably, the child died because we asked it to (sr->service_state == STOPPING). But even if we
@@ -235,16 +236,17 @@ dasynq::rearm service_child_watcher::status_change(eventloop_t &loop, pid_t chil
         sr->waiting_stopstart_timer = false;
     }
 
-    sr->handle_exit_status(bp_sys::exit_status(status));
+    sr->handle_exit_status();
     return dasynq::rearm::NOOP;
 }
 
-dasynq::rearm stop_child_watcher::status_change(eventloop_t &loop, pid_t child, int status) noexcept
+dasynq::rearm stop_child_watcher::status_change(eventloop_t &loop, pid_t child,
+        stop_child_watcher::proc_status_t status) noexcept
 {
     process_service *sr = service;
 
     sr->stop_pid = -1;
-    sr->stop_status = bp_sys::exit_status(status);
+    sr->stop_status = status;
     stop_watch(loop);
 
     if (sr->waiting_for_execstat) {
@@ -314,7 +316,7 @@ rearm log_output_watcher::fd_event(eventloop_t &eloop, int fd, int flags) noexce
     return rearm::REMOVED;
 }
 
-void process_service::handle_exit_status(bp_sys::exit_status exit_status) noexcept
+void process_service::handle_exit_status() noexcept
 {
     bool did_exit = exit_status.did_exit();
     bool was_signalled = exit_status.was_signalled();
@@ -333,7 +335,7 @@ void process_service::handle_exit_status(bp_sys::exit_status exit_status) noexce
         }
         else if (was_signalled) {
             log(loglevel_t::ERROR, "Service ", get_name(), " terminated due to signal ",
-                    exit_status.get_term_sig());
+                    exit_status.get_signal());
         }
     }
 
@@ -416,7 +418,7 @@ void process_service::exec_failed(run_proc_err errcode) noexcept
     }
 }
 
-void bgproc_service::handle_exit_status(bp_sys::exit_status exit_status) noexcept
+void bgproc_service::handle_exit_status() noexcept
 {
     // For bgproc services, receiving exit status can mean one of two things:
     // 1. We were launching the process, and it finished (possibly after forking). If it did fork
@@ -437,7 +439,7 @@ void bgproc_service::handle_exit_status(bp_sys::exit_status exit_status) noexcep
         }
         else if (was_signalled) {
             log(loglevel_t::ERROR, "Service ", get_name(), " terminated due to signal ",
-                    exit_status.get_term_sig());
+                    exit_status.get_signal());
         }
     }
 
@@ -548,6 +550,7 @@ void bgproc_service::handle_exit_status(bp_sys::exit_status exit_status) noexcep
         // We won't log a non-zero exit status or termination due to signal here -
         // we assume that the process died because we signalled it.
         if (stop_pid == -1 && !waiting_for_execstat) {
+            stop_issued = false;
             stopped();
         }
     }
@@ -589,7 +592,7 @@ void bgproc_service::exec_failed(run_proc_err errcode) noexcept
     }
 }
 
-void scripted_service::handle_exit_status(bp_sys::exit_status exit_status) noexcept
+void scripted_service::handle_exit_status() noexcept
 {
     bool did_exit = exit_status.did_exit();
     bool was_signalled = exit_status.was_signalled();
@@ -618,7 +621,7 @@ void scripted_service::handle_exit_status(bp_sys::exit_status exit_status) noexc
             else {
                 if (was_signalled) {
                     log(loglevel_t::NOTICE, "Service ", get_name(), " start cancelled from signal ",
-                            exit_status.get_term_sig());
+                            exit_status.get_signal());
                 }
                 // If the start script completed successfully, or was interrupted via our signal,
                 // we want to run the stop script to clean up:
@@ -638,7 +641,7 @@ void scripted_service::handle_exit_status(bp_sys::exit_status exit_status) noexc
             }
             else if (was_signalled) {
                 log(loglevel_t::WARN, "Service ", get_name(), " stop command terminated due to signal ",
-                        exit_status.get_term_sig());
+                        exit_status.get_signal());
             }
             // Even if the stop script failed, assume that service is now stopped, so that any dependencies
             // can be stopped. There's not really any other useful course of action here.
@@ -650,7 +653,7 @@ void scripted_service::handle_exit_status(bp_sys::exit_status exit_status) noexc
         if (exit_status.did_exit_clean()) {
             started();
         }
-        else if (was_signalled && exit_status.get_term_sig() == SIGINT && onstart_flags.skippable) {
+        else if (was_signalled && exit_status.get_signal() == SIGINT && onstart_flags.skippable) {
             // A skippable service can be skipped by interrupting (eg by ^C if the service
             // starts on the console).
             start_skipped = true;
@@ -664,7 +667,7 @@ void scripted_service::handle_exit_status(bp_sys::exit_status exit_status) noexc
             }
             else if (was_signalled) {
                 log(loglevel_t::ERROR, "Service ", get_name(), " command terminated due to signal ",
-                        exit_status.get_term_sig());
+                        exit_status.get_signal());
             }
             stop_reason = stopped_reason_t::FAILED;
             failed_to_start();
@@ -696,7 +699,7 @@ template <typename T> typename std::make_unsigned<T>::type make_unsigned_val(T v
 }
 
 bgproc_service::pid_result_t
-bgproc_service::read_pid_file(bp_sys::exit_status *exit_status) noexcept
+bgproc_service::read_pid_file(proc_status_t *exit_status) noexcept
 {
     const char *pid_file_c = pid_file.c_str();
     int fd = bp_sys::open(pid_file_c, O_CLOEXEC);
@@ -718,40 +721,40 @@ bgproc_service::read_pid_file(bp_sys::exit_status *exit_status) noexcept
     pidbuf[r] = 0; // store nul terminator
 
     bool valid_pid = false;
-    try {
-        unsigned long long v = std::strtoull(pidbuf, nullptr, 0);
+    errno = 0;
+    char *eptr;
+    unsigned long long v = std::strtoull(pidbuf, &eptr, 0);
+    if (errno == 0 && eptr != pidbuf) {
         if (v <= make_unsigned_val(std::numeric_limits<pid_t>::max())) {
             pid = (pid_t) v;
             valid_pid = true;
         }
     }
-    catch (std::out_of_range &exc) {
-        // Too large?
-    }
-    catch (std::invalid_argument &exc) {
-        // Ok, so it doesn't look like a number: proceed...
-    }
 
     if (valid_pid) {
-        pid_t wait_r = waitpid(pid, exit_status, WNOHANG);
-        if (wait_r == -1 && errno == ECHILD) {
+        siginfo_t child_info;
+        child_info.si_pid = 0; // for portability
+        int waitid_r = bp_sys::waitid(P_PID, pid, &child_info, WNOHANG | WEXITED);
+        if (waitid_r == -1 && errno == ECHILD) {
             // We can't track this child - check process exists:
             if (bp_sys::kill(pid, 0) == 0 || errno != ESRCH) {
+                log(loglevel_t::WARN, get_name(), ": unable to supervise process with pid ", pid);
                 tracking_child = false;
                 return pid_result_t::OK;
             }
-            else {
-                log(loglevel_t::ERROR, get_name(), ": pid read from pidfile (", pid, ") is not valid");
-                pid = -1;
-                return pid_result_t::FAILED;
-            }
-        }
-        else if (wait_r == pid) {
+
+            log(loglevel_t::ERROR, get_name(), ": pid read from pidfile (", pid, ") is not valid");
             pid = -1;
-            return pid_result_t::TERMINATED;
+            return pid_result_t::FAILED;
         }
-        else if (wait_r == 0) {
-            // We can track the child
+        else if (waitid_r == 0) {
+            if (child_info.si_pid != 0) {
+                pid = -1;
+                *exit_status = proc_status_t(child_info.si_code, child_info.si_status);
+                return pid_result_t::TERMINATED;
+            }
+
+            // Child hasn't terminated, but we can track it (no error)
             child_listener.add_reserved(event_loop, pid, dasynq::DEFAULT_PRIORITY - 10);
             tracking_child = true;
             reserved_child_watch = true;
