@@ -18,6 +18,7 @@
 #include "cpbuffer.h"
 #include "control-cmds.h"
 #include "service-constants.h"
+#include "static-string.h"
 #include "dinit-client.h"
 #include "dinit-util.h"
 #include "mconfig.h"
@@ -29,7 +30,10 @@
 // This utility communicates with the dinit daemon via a unix socket (specified in SYSCONTROLSOCKET).
 
 static constexpr uint16_t min_cp_version = 1;
-static constexpr uint16_t max_cp_version = 1;
+static constexpr uint16_t max_cp_version = 5;
+
+static constexpr auto reboot_execname = cts::literal(SHUTDOWN_PREFIX) + cts::literal("reboot");
+static constexpr auto soft_reboot_execname = cts::literal(SHUTDOWN_PREFIX) + cts::literal("soft-reboot");
 
 using loop_t = dasynq::event_loop_n;
 using rearm = dasynq::rearm;
@@ -39,7 +43,8 @@ class subproc_buffer;
 void do_system_shutdown(shutdown_type_t shutdown_type);
 static void unmount_disks(loop_t &loop, subproc_buffer &sub_buf);
 static void swap_off(loop_t &loop, subproc_buffer &sub_buf);
-static int run_process(const char * prog_args[], loop_t &loop, subproc_buffer &sub_buf);
+static loop_t::child_proc_watcher::proc_status_t run_process(const char * prog_args[],
+        loop_t &loop, subproc_buffer &sub_buf);
 
 constexpr static int subproc_bufsize = 4096;
 
@@ -253,9 +258,12 @@ int main(int argc, char **argv)
     auto shutdown_type = shutdown_type_t::POWEROFF;
 
     const char *execname = base_name(argv[0]);
-    if (strcmp(execname, "reboot") == 0) {
+    if (strcmp(execname, reboot_execname) == 0) {
         shutdown_type = shutdown_type_t::REBOOT;
     }
+    else if (strcmp(execname, soft_reboot_execname) == 0) {
+        shutdown_type = shutdown_type_t::SOFTREBOOT;
+    }
         
     for (int i = 1; i < argc; i++) {
         if (argv[i][0] == '-') {
@@ -276,6 +284,9 @@ int main(int argc, char **argv)
             else if (strcmp(argv[i], "-p") == 0) {
                 shutdown_type = shutdown_type_t::POWEROFF;
             }
+            else if (strcmp(argv[i], "-s") == 0) {
+                shutdown_type = shutdown_type_t::SOFTREBOOT;
+            }
             else if (strcmp(argv[i], "--use-passed-cfd") == 0) {
                 use_passed_cfd = true;
             }
@@ -294,6 +305,7 @@ int main(int argc, char **argv)
         cout << execname << " :   shutdown the system\n"
                 "  --help           : show this help\n"
                 "  -r               : reboot\n"
+                "  -s               : soft-reboot (restart dinit with same boot-time arguments)\n"
                 "  -h               : halt system\n"
                 "  -p               : power down (default)\n"
                 "  --use-passed-cfd : use the socket file descriptor identified by the DINIT_CS_FD\n"
@@ -382,7 +394,7 @@ int main(int argc, char **argv)
         cerr << "shutdown: control socket write error: " << std::strerror(e.errcode) << endl;
         return 1;
     }
-    
+
     while (true) {
         pause();
     }
@@ -458,8 +470,8 @@ void do_system_shutdown(shutdown_type_t shutdown_type)
             sub_buf.append("Executing shutdown hook...\n");
             const char *prog_args[] = { hook_paths[i], nullptr };
             try {
-                int r = run_process(prog_args, loop, sub_buf);
-                if (r == 0) {
+                auto r = run_process(prog_args, loop, sub_buf);
+                if (r.did_exit() && r.get_exit_status() == 0) {
                     do_unmount_ourself = false;
                 }
             }
@@ -542,15 +554,16 @@ class subproc_out_watch : public loop_t::fd_watcher_impl<subproc_out_watch>
 
 // Run process, put its output through the subprocess buffer
 //   may throw: std::system_error, std::bad_alloc
-static int run_process(const char * prog_args[], loop_t &loop, subproc_buffer &sub_buf)
+static loop_t::child_proc_watcher::proc_status_t run_process(const char * prog_args[],
+        loop_t &loop, subproc_buffer &sub_buf)
 {
     class sp_watcher_t : public loop_t::child_proc_watcher_impl<sp_watcher_t>
     {
         public:
         bool terminated = false;
-        int exit_status;
+        proc_status_t exit_status;
 
-        rearm status_change(loop_t &, pid_t child, int status)
+        rearm status_change(loop_t &, pid_t child, proc_status_t status)
         {
             terminated = true;
             exit_status = status;
